/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.admin.amx.mbean;

import com.sun.appserv.management.base.AMX;

import javax.management.ObjectName;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
	Base class from which all AMX MBeans should derive which are NOT
	Config-related.
	<p>
 */
public abstract class AMXNonConfigImplBase extends AMXImplBase
{
    private final Set<ObjectName>   mChildren = new HashSet<ObjectName>();
    
		protected
	AMXNonConfigImplBase(
        final String j2eeType,
        final String fullType,
        final ObjectName parentObjectName,
		final Class<? extends AMX> theInterface,
        final Delegate delegate )
	{
		super( j2eeType, fullType, parentObjectName, theInterface, delegate );
	}
    
    @Override
		protected void
	postRegisterHook( final Boolean registrationSucceeded )
	{
        super.postRegisterHook( registrationSucceeded );
	    if ( registrationSucceeded.booleanValue() )
		{
            registerChildren();
		}
	}
    
    @Override
        protected void
	preDeregisterHook()
		throws Exception
	{
        super.preDeregisterHook();
        
		unregisterChildren( );
	}
	
    @Override
        protected void
	postDeregisterHook()
	{
	}
    
        protected void
	registerChildren()
	{
	}
    
        protected synchronized ObjectName
    registerChild( final Object mbean, final ObjectName childObjectName )
    {
        try
        {
            final ObjectName objectName = getMBeanServer().registerMBean( mbean, childObjectName ).getObjectName();
        
            return objectName;
        }
        catch ( Exception e )
        {
            throw new RuntimeException(e);
        }
    }
    
        protected synchronized Set<ObjectName>
    getChildren()
    {
        return Collections.unmodifiableSet( mChildren );
    }
    
		protected synchronized void
	unregisterChildren()
	{
        for ( final ObjectName childObjectName : mChildren )
        {
            try
            {
                getMBeanServer().unregisterMBean( childObjectName );
                debug( "unregisterSelfMgrChildren: ", childObjectName, " is now unregistered" );
            }
            catch ( Exception e )
            {
                // continue even if one cannot be unregistered
                // throw new RuntimeException(e);
            }
        }
        mChildren.clear();
	}
}




















